@@ -11,11 +11,32 @@ import android.util.Log;
 import android.widget.Toast;
 
 public class ForegroundService extends Service {
+    private static final String TAG="FgService";
+
+    private volatile boolean shown;
+
+    private final DaemonSingleton.StateUpdateListener daemonStateUpdatedListener =
+            new DaemonSingleton.StateUpdateListener() {
+
+                @Override
+                public void daemonStateUpdate() {
+                    try {
+                        synchronized (ForegroundService.this) {
+                            if (shown) cancelNotification();
+                            showNotification();
+                        }
+                    } catch (Throwable tr) {
+                        Log.e(TAG,"error ignored",tr);
+                    }
+                }
+            };
+
+
     private NotificationManager notificationManager;
 
     // Unique Identification Number for the Notification.
     // We use it on Notification start, and to cancel it.
-    private int NOTIFICATION = R.string.i2pd_started;
+    private int NOTIFICATION = 1;
 
     /**
      * Class for clients to access.  Because we know this service always
@@ -32,29 +53,35 @@ public class ForegroundService extends Service {
     public void onCreate() {
         notificationManager = (NotificationManager)getSystemService(NOTIFICATION_SERVICE);
 
-        // Display a notification about us starting.  We put an icon in the status bar.
-        showNotification();
-        daemon.start();
+        synchronized (this) {
+            DaemonSingleton.getInstance().addStateChangeListener(daemonStateUpdatedListener);
+            if (!shown) daemonStateUpdatedListener.daemonStateUpdate();
+        }
         // Tell the user we started.
-        Toast.makeText(this, R.string.i2pd_service_started, Toast.LENGTH_SHORT).show();
+//        Toast.makeText(this, R.string.i2pd_service_started, Toast.LENGTH_SHORT).show();
     }
 
     @Override
     public int onStartCommand(Intent intent, int flags, int startId) {
         Log.i("ForegroundService", "Received start id " + startId + ": " + intent);
-        daemon.start();
         return START_STICKY;
     }
 
     @Override
     public void onDestroy() {
+        DaemonSingleton.getInstance().removeStateChangeListener(daemonStateUpdatedListener);
+        cancelNotification();
+    }
+
+    private synchronized void cancelNotification() {
         // Cancel the persistent notification.
         notificationManager.cancel(NOTIFICATION);
-        
+
         stopForeground(true);
 
         // Tell the user we stopped.
-        Toast.makeText(this, R.string.i2pd_service_stopped, Toast.LENGTH_SHORT).show();
+//        Toast.makeText(this, R.string.i2pd_service_stopped, Toast.LENGTH_SHORT).show();
+        shown=false;
     }
 
     @Override
@@ -69,13 +96,13 @@ public class ForegroundService extends Service {
     /**
      * Show a notification while this service is running.
      */
-    private void showNotification() {
+    private synchronized void showNotification() {
         // In this sample, we'll use the same text for the ticker and the expanded notification
-        CharSequence text = getText(R.string.i2pd_started);
+        CharSequence text = getText(DaemonSingleton.getInstance().getState().getStatusStringResourceId());
 
         // The PendingIntent to launch our activity if the user selects this notification
         PendingIntent contentIntent = PendingIntent.getActivity(this, 0,
-                new Intent(this, I2PD.class), 0);
+                new Intent(this, I2PDActivity.class), 0);
 
         // Set the info for the views that show in the notification panel.
         Notification notification = new Notification.Builder(this)
@@ -90,8 +117,9 @@ public class ForegroundService extends Service {
         // Send the notification.
         //mNM.notify(NOTIFICATION, notification);
         startForeground(NOTIFICATION, notification);
+        shown=true;
     }
-    
-	private final DaemonSingleton daemon = DaemonSingleton.getInstance();
+
+	private static final DaemonSingleton daemon = DaemonSingleton.getInstance();
 }
 
