@@ -1,6 +1,8 @@
 #include <fstream>
+#include <openssl/rand.h>
 #include "Config.h"
 #include "Crypto.h"
+#include "Ed25519.h"
 #include "Timestamp.h"
 #include "I2NPProtocol.h"
 #include "NetDb.hpp"
@@ -28,18 +30,14 @@ namespace i2p
 		m_StartupTime = i2p::util::GetSecondsSinceEpoch ();
 		if (!Load ())
 			CreateNewRouter ();
-		m_Decryptor = m_Keys.CreateDecryptor (nullptr); 
+		m_Decryptor = m_Keys.CreateDecryptor (nullptr);
 		UpdateRouterInfo ();
 	}
 
 	void RouterContext::CreateNewRouter ()
 	{
-#if defined(__x86_64__) || defined(__i386__) || defined(_MSC_VER)
 		m_Keys = i2p::data::PrivateKeys::CreateRandomKeys (i2p::data::SIGNING_KEY_TYPE_EDDSA_SHA512_ED25519);
-#else
-		m_Keys = i2p::data::PrivateKeys::CreateRandomKeys (i2p::data::SIGNING_KEY_TYPE_DSA_SHA1);
-#endif
-		SaveKeys ();
+		SaveKeys ();			
 		NewRouterInfo ();
 	}
 
@@ -52,7 +50,8 @@ namespace i2p
 			port = rand () % (30777 - 9111) + 9111; // I2P network ports range
 		bool ipv4; i2p::config::GetOption("ipv4", ipv4);
 		bool ipv6; i2p::config::GetOption("ipv6", ipv6);
-		bool nat;  i2p::config::GetOption("nat", nat);
+		bool ntcp2;  i2p::config::GetOption("ntcp2", ntcp2);
+		bool nat;  i2p::config::GetOption("nat", nat);	
 		std::string ifname; i2p::config::GetOption("ifname", ifname);
 		std::string ifname4; i2p::config::GetOption("ifname4", ifname4);
 		std::string ifname6; i2p::config::GetOption("ifname6", ifname6);
@@ -93,6 +92,12 @@ namespace i2p
 		routerInfo.CreateBuffer (m_Keys);
 		m_RouterInfo.SetRouterIdentity (GetIdentity ());
 		m_RouterInfo.Update (routerInfo.GetBuffer (), routerInfo.GetBufferLen ());
+
+		if (ntcp2)
+		{ 
+			NewNTCP2Keys ();
+			m_RouterInfo.AddNTCP2Address (m_NTCP2Keys->staticPublicKey, m_NTCP2Keys->iv);
+		}
 	}
 
 	void RouterContext::UpdateRouterInfo ()
@@ -102,6 +107,19 @@ namespace i2p
 		m_LastUpdateTime = i2p::util::GetSecondsSinceEpoch ();
 	}
 
+	void RouterContext::NewNTCP2Keys ()
+	{
+		m_NTCP2Keys.reset (new NTCP2PrivateKeys ());
+		RAND_bytes (m_NTCP2Keys->staticPrivateKey, 32);
+		RAND_bytes (m_NTCP2Keys->iv, 16);
+		BN_CTX * ctx = BN_CTX_new ();
+		i2p::crypto::GetEd25519 ()->ScalarMulB (m_NTCP2Keys->staticPrivateKey, m_NTCP2Keys->staticPublicKey, ctx); 
+		BN_CTX_free (ctx);
+		// save
+		std::ofstream fk (i2p::fs::DataDirPath (NTCP2_KEYS), std::ofstream::binary | std::ofstream::out);
+		fk.write ((char *)m_NTCP2Keys.get (), sizeof (NTCP2PrivateKeys)); 
+	}
+
 	void RouterContext::SetStatus (RouterStatus status)
 	{
 		if (status != m_Status)
@@ -204,7 +222,7 @@ namespace i2p
 		}
 	}
 
-	void RouterContext::SetBandwidth (char L) 
+	void RouterContext::SetBandwidth (char L)
 	{
 		uint32_t limit = 0;
 		enum { low, high, extra, unlim } type = high;
@@ -433,6 +451,30 @@ namespace i2p
 		if (IsUnreachable ())
 			SetReachable (); // we assume reachable until we discover firewall through peer tests
 
+		// read NTCP2
+		bool ntcp2;  i2p::config::GetOption("ntcp2", ntcp2);
+		if (ntcp2)
+		{
+			std::ifstream n2k (i2p::fs::DataDirPath (NTCP2_KEYS), std::ifstream::in | std::ifstream::binary);
+			if (n2k) 
+			{
+				n2k.seekg (0, std::ios::end);
+				len = fk.tellg();
+				n2k.seekg (0, std::ios::beg);
+				if (len == sizeof (NTCP2PrivateKeys))
+				{
+					m_NTCP2Keys.reset (new NTCP2PrivateKeys ());
+					n2k.read ((char *)m_NTCP2Keys.get (), sizeof (NTCP2PrivateKeys));				
+				}	
+				n2k.close ();
+			}
+			if (!m_NTCP2Keys) 
+			{
+				NewNTCP2Keys ();
+				m_RouterInfo.AddNTCP2Address (m_NTCP2Keys->staticPublicKey, m_NTCP2Keys->iv);
+			}
+		}
+
 		return true;
 	}
 
@@ -482,6 +524,11 @@ namespace i2p
 
 	bool RouterContext::Decrypt (const uint8_t * encrypted, uint8_t * data, BN_CTX * ctx) const
 	{
-		return m_Decryptor ? m_Decryptor->Decrypt (encrypted, data, ctx) : false;
+		return m_Decryptor ? m_Decryptor->Decrypt (encrypted, data, ctx, true) : false;
+	}
+
+	bool RouterContext::DecryptTunnelBuildRecord (const uint8_t * encrypted, uint8_t * data, BN_CTX * ctx) const
+	{
+		return m_Decryptor ? m_Decryptor->Decrypt (encrypted, data, ctx, false) : false;
 	}
 }
