@@ -8,7 +8,7 @@
 #include <list>
 #include <iostream>
 #include <boost/asio.hpp>
-#include <boost/shared_ptr.hpp> 
+#include <boost/shared_ptr.hpp>
 #include "Identity.h"
 #include "Profiling.h"
 
@@ -17,15 +17,15 @@ namespace i2p
 namespace data
 {
 	const char ROUTER_INFO_PROPERTY_LEASESETS[] = "netdb.knownLeaseSets";
-	const char ROUTER_INFO_PROPERTY_ROUTERS[] = "netdb.knownRouters";	
+	const char ROUTER_INFO_PROPERTY_ROUTERS[] = "netdb.knownRouters";
 	const char ROUTER_INFO_PROPERTY_NETID[] = "netId";
-	const char ROUTER_INFO_PROPERTY_FAMILY[] = "family";	
+	const char ROUTER_INFO_PROPERTY_FAMILY[] = "family";
 	const char ROUTER_INFO_PROPERTY_FAMILY_SIG[] = "family.sig";
-	
+
 	const char CAPS_FLAG_FLOODFILL = 'f';
 	const char CAPS_FLAG_HIDDEN = 'H';
 	const char CAPS_FLAG_REACHABLE = 'R';
-	const char CAPS_FLAG_UNREACHABLE = 'U';	
+	const char CAPS_FLAG_UNREACHABLE = 'U';
 	/* bandwidth flags */
 	const char CAPS_FLAG_LOW_BANDWIDTH1   = 'K'; /*   < 12 KBps */
 	const char CAPS_FLAG_LOW_BANDWIDTH2   = 'L'; /*  12-48 KBps */
@@ -34,7 +34,7 @@ namespace data
 	const char CAPS_FLAG_HIGH_BANDWIDTH3  = 'O'; /* 128-256 KBps */
 	const char CAPS_FLAG_EXTRA_BANDWIDTH1 = 'P'; /* 256-2000 KBps */
 	const char CAPS_FLAG_EXTRA_BANDWIDTH2 = 'X'; /*   > 2000 KBps */
-	
+
 	const char CAPS_FLAG_SSU_TESTING = 'B';
 	const char CAPS_FLAG_SSU_INTRODUCER = 'C';
 
@@ -44,13 +44,15 @@ namespace data
 		public:
 
 			enum SupportedTranports
-			{	
+			{
 				eNTCPV4 = 0x01,
 				eNTCPV6 = 0x02,
 				eSSUV4 = 0x04,
-				eSSUV6 = 0x08
+				eSSUV6 = 0x08,
+				eNTCP2V4 = 0x10,
+				eNTCP2V6 = 0x20
 			};
-			
+
 			enum Caps
 			{
 				eFloodfill = 0x01,
@@ -71,7 +73,7 @@ namespace data
 			};
 
 			typedef Tag<32> IntroKey; // should be castable to MacKey and AESKey
-			struct Introducer			
+			struct Introducer
 			{
 				Introducer (): iExp (0) {};
 				boost::asio::ip::address iHost;
@@ -85,9 +87,15 @@ namespace data
 			{
 				int mtu;
 				IntroKey key; // intro key for SSU
-				std::vector<Introducer> introducers;		
+				std::vector<Introducer> introducers;
 			};
-			
+
+			struct NTCP2Ext
+			{
+				Tag<32> staticKey;
+				Tag<16> iv;
+			};
+
 			struct Address
 			{
 				TransportStyle transportStyle;
@@ -97,24 +105,27 @@ namespace data
 				uint64_t date;
 				uint8_t cost;
 				std::unique_ptr<SSUExt> ssu; // not null for SSU
+				std::unique_ptr<NTCP2Ext> ntcp2; // not null for NTCP2
 
-				bool IsCompatible (const boost::asio::ip::address& other) const 
+				bool IsCompatible (const boost::asio::ip::address& other) const
 				{
 					return (host.is_v4 () && other.is_v4 ()) ||
 						(host.is_v6 () && other.is_v6 ());
-				}	
+				}
 
 				bool operator==(const Address& other) const
 				{
 					return transportStyle == other.transportStyle && host == other.host && port == other.port;
-				}	
+				}
 
 				bool operator!=(const Address& other) const
 				{
 					return !(*this == other);
-				}	
+				}
+
+				bool IsNTCP2 () const { return (bool)ntcp2; };
 			};
-			typedef std::list<std::shared_ptr<Address> > Addresses;			
+			typedef std::list<std::shared_ptr<Address> > Addresses;
 
 			RouterInfo ();
 			RouterInfo (const std::string& fullPath);
@@ -122,7 +133,7 @@ namespace data
 			RouterInfo& operator=(const RouterInfo& ) = default;
 			RouterInfo (const uint8_t * buf, int len);
 			~RouterInfo ();
-			
+
 			std::shared_ptr<const IdentityEx> GetRouterIdentity () const { return m_RouterIdentity; };
 			void SetRouterIdentity (std::shared_ptr<const IdentityEx> identity);
 			std::string GetIdentHashBase64 () const { return GetIdentHash ().ToBase64 (); };
@@ -131,9 +142,10 @@ namespace data
 			std::shared_ptr<const Address> GetNTCPAddress (bool v4only = true) const;
 			std::shared_ptr<const Address> GetSSUAddress (bool v4only = true) const;
 			std::shared_ptr<const Address> GetSSUV6Address () const;
-			
+
 			void AddNTCPAddress (const char * host, int port);
 			void AddSSUAddress (const char * host, int port, const uint8_t * key, int mtu = 0);
+			void AddNTCP2Address (const uint8_t * staticKey, const uint8_t * iv);
 			bool AddIntroducer (const Introducer& introducer);
 			bool RemoveIntroducer (const boost::asio::ip::udp::endpoint& e);
 			void SetProperty (const std::string& key, const std::string& value); // called from RouterContext only
@@ -144,6 +156,7 @@ namespace data
 			bool IsReachable () const { return m_Caps & Caps::eReachable; };
 			bool IsNTCP (bool v4only = true) const;
 			bool IsSSU (bool v4only = true) const;
+			bool IsNTCP2 (bool v4only = true) const;
 			bool IsV6 () const;
 			bool IsV4 () const;
 			void EnableV6 ();
@@ -156,37 +169,37 @@ namespace data
 			bool IsPeerTesting () const { return m_Caps & eSSUTesting; };
 			bool IsHidden () const { return m_Caps & eHidden; };
 			bool IsHighBandwidth () const { return m_Caps & RouterInfo::eHighBandwidth; };
-			bool IsExtraBandwidth () const { return m_Caps & RouterInfo::eExtraBandwidth; };	
-			
-			uint8_t GetCaps () const { return m_Caps; };	
+			bool IsExtraBandwidth () const { return m_Caps & RouterInfo::eExtraBandwidth; };
+
+			uint8_t GetCaps () const { return m_Caps; };
 			void SetCaps (uint8_t caps);
 			void SetCaps (const char * caps);
 
-			void SetUnreachable (bool unreachable) { m_IsUnreachable = unreachable; }; 
+			void SetUnreachable (bool unreachable) { m_IsUnreachable = unreachable; };
 			bool IsUnreachable () const { return m_IsUnreachable; };
 
 			const uint8_t * GetBuffer () const { return m_Buffer; };
 			const uint8_t * LoadBuffer (); // load if necessary
-			int GetBufferLen () const { return m_BufferLen; };			
+			int GetBufferLen () const { return m_BufferLen; };
 			void CreateBuffer (const PrivateKeys& privateKeys);
 
 			bool IsUpdated () const { return m_IsUpdated; };
-			void SetUpdated (bool updated) { m_IsUpdated = updated; }; 
+			void SetUpdated (bool updated) { m_IsUpdated = updated; };
 			bool SaveToFile (const std::string& fullPath);
 
 			std::shared_ptr<RouterProfile> GetProfile () const;
 			void SaveProfile () { if (m_Profile) m_Profile->Save (GetIdentHash ()); };
-			
+
 			void Update (const uint8_t * buf, int len);
 			void DeleteBuffer () { delete[] m_Buffer; m_Buffer = nullptr; };
-			bool IsNewer (const uint8_t * buf, size_t len) const;			
+			bool IsNewer (const uint8_t * buf, size_t len) const;
+
+		/** return true if we are in a router family and the signature is valid */
+		bool IsFamily(const std::string & fam) const;
 
-     	 	/** return true if we are in a router family and the signature is valid */
-      		bool IsFamily(const std::string & fam) const;
-      
 			// implements RoutingDestination
 			std::shared_ptr<const IdentityEx> GetIdentity () const { return m_RouterIdentity; };
-			void Encrypt (const uint8_t * data, uint8_t * encrypted, BN_CTX * ctx) const;			
+			void Encrypt (const uint8_t * data, uint8_t * encrypted, BN_CTX * ctx) const;
 
 			bool IsDestination () const { return false; };
 
@@ -201,7 +214,7 @@ namespace data
 			void WriteString (const std::string& str, std::ostream& s) const;
 			void ExtractCaps (const char * value);
 			std::shared_ptr<const Address> GetAddress (TransportStyle s, bool v4only, bool v6only = false) const;
-			void UpdateCapsProperty ();			
+			void UpdateCapsProperty ();
 
 		private:
 
@@ -210,13 +223,13 @@ namespace data
 			uint8_t * m_Buffer;
 			size_t m_BufferLen;
 			uint64_t m_Timestamp;
-			boost::shared_ptr<Addresses> m_Addresses; // TODO: use std::shared_ptr and std::atomic_store for gcc >= 4.9 
+			boost::shared_ptr<Addresses> m_Addresses; // TODO: use std::shared_ptr and std::atomic_store for gcc >= 4.9
 			std::map<std::string, std::string> m_Properties;
 			bool m_IsUpdated, m_IsUnreachable;
 			uint8_t m_SupportedTransports, m_Caps;
 			mutable std::shared_ptr<RouterProfile> m_Profile;
-	};	
-}	
+	};
+}
 }
 
 #endif
